/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <tizen.h>
#include "main.h"

typedef struct appdata {
	Evas_Object *win;
	Evas_Object *nf;
} appdata_s;

typedef struct _item_data_s
{
	int index;
	Elm_Object_Item *item;
} item_data_s;

char *genlist_times[] = {"10:00 AM", "10:01 AM", "10:02 AM", "10:03 AM", "10:04 AM", "10:05 AM",
	"3:50 PM", "3:51 PM", "3:52 PM", "3:53 PM"};

char *genlist_messages[] = {"Good morning!",
	"It's you again. How do you do this?",
	"This is the princess who fell from the sky into my arms.",
	"Yes. We keep meeting like this. You just suddenly show up.",
	"We could make plans to meet.",
	"No, it's nicer this way. I hope we meet again suddenly.",
	"Don't you know that it takes so little to make me happy?",
	"Good morning, Princess!",
	"This is incredible. You owe me an explanation.",
	"No, you're the one who owes me an explanation. You've really got a crush on me. Where shall we go, Princess?"
};

static void
win_delete_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

static Evas_Object *
create_label(appdata_s *ad, const char *text)
{
	Evas_Object *label;

	label = elm_label_add(ad->nf);
	elm_object_text_set(label, text);
	elm_label_line_wrap_set(label, ELM_WRAP_WORD);
	elm_label_wrap_width_set(label, 300);
	evas_object_size_hint_weight_set(label, 0.5, 0.5);
	evas_object_size_hint_align_set(label, EVAS_HINT_FILL, EVAS_HINT_FILL);

	evas_object_show(label);

	return label;
}

static char *
gl_text_get(void *data, Evas_Object *obj, const char *part)
{
	item_data_s *id = data;
	int index = id->index % 10;

	if (id->index == 0)
		return strdup("sam@samsung.com");

	if (!strcmp(part, "elm.text")) {
		return strdup(genlist_messages[index]);
	} else if (!strcmp(part, "elm.text.1") ) {
		return strdup(genlist_times[index]);
	}
	return NULL;
}

static Evas_Object *
gl_icon_get(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *icon = NULL;

	if (!strcmp(part, "elm.swallow.colorbar")) {
		icon = evas_object_rectangle_add(evas_object_evas_get(obj));
		evas_object_color_set(icon, 255, 0, 0, 255);
		evas_object_size_hint_min_set(icon, 7 * elm_config_scale_get(), 0);
		evas_object_size_hint_max_set(icon, 7 * elm_config_scale_get(), -1);
		return icon;
	}
	return icon;
}

static void
gl_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *label;
	Elm_Object_Item *it = event_info;
	appdata_s *ad = data;
	elm_genlist_item_selected_set(it, EINA_FALSE);
	label = create_label(ad, elm_object_item_part_text_get(it, "elm.text"));
	elm_naviframe_item_push(ad->nf, elm_object_item_part_text_get(it, "elm.text.1"), NULL, NULL, label, NULL);
}

static void
gl_del_cb(void *data, Evas_Object *obj)
{
	item_data_s *id = data;
	if (id) free(id);
}

static void
block_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del(obj);
}

static void
timeout_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del(obj);
}

static void
clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = data;
	Evas_Object *popup;

	popup = elm_popup_add(ad->win);
	elm_object_style_set(popup, "toast");
	elm_popup_orient_set(popup, ELM_POPUP_ORIENT_BOTTOM);
	elm_popup_timeout_set(popup, 1.5);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);
	elm_object_part_text_set(popup,"elm.text", "Sample!");

	evas_object_smart_callback_add(popup, "block,clicked", block_clicked_cb, NULL);
	evas_object_smart_callback_add(popup, "timeout", timeout_cb, NULL);

	evas_object_show(popup);
}

static Evas_Object*
create_title_btn(appdata_s *ad)
{
	Evas_Object *ic;
	Evas_Object *nf = ad->nf;
	char buf[PATH_MAX];
	Evas_Object *btn = elm_button_add(nf);
	if (!btn) return NULL;
	elm_object_style_set(btn, "naviframe/title_icon");
	ic = elm_image_add(btn);
	snprintf(buf, sizeof(buf), "%s/%s", ICON_DIR, "tw_ic_menu_delete_holo_dark.png");
	elm_image_file_set(ic, buf, NULL);
	evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_BOTH, 1, 1);
	elm_image_resizable_set(ic, EINA_TRUE, EINA_TRUE);
	elm_object_part_content_set(btn, "icon", ic);
	evas_object_smart_callback_add(btn, "clicked", clicked_cb, ad);
	return btn;
}

static Evas_Object*
create_genlist(appdata_s *ad)
{
	Evas_Object *genlist;
	Elm_Object_Item *item;
	int index = 0;

	/* Create item class */
	Elm_Genlist_Item_Class *gtc = elm_genlist_item_class_new();
	Elm_Genlist_Item_Class *itc = elm_genlist_item_class_new();

	gtc->item_style = "title";
	gtc->func.text_get = gl_text_get;
	gtc->func.content_get = gl_icon_get;
	gtc->func.del = gl_del_cb;

	itc->item_style = "2text";
	itc->func.text_get = gl_text_get;
	itc->func.content_get = gl_icon_get;
	itc->func.del = gl_del_cb;

	/* Genlist */
	genlist = elm_genlist_add(ad->nf);
	/* COMPRESS MODE
		For the Mobile view, because it has full window, compresse mode should be used. */
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	/* HOMOGENEOUS MODE
		If item height is same when each style name is same, Use homogeneous mode. */
	elm_genlist_homogeneous_set(genlist, EINA_TRUE);

	for (index = 0; index < 20; index++) {
		item_data_s *id = calloc(sizeof(item_data_s), 1);
		id->index = index;
		if (index == 0)	{
			item = elm_genlist_item_append(
					genlist,				// genlist object
					gtc,					// item class
					id,						// item class user data
					NULL,
					ELM_GENLIST_ITEM_GROUP,	// item type
					gl_selected_cb,				// select smart callback
					ad);                    // smart callback user data
			elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
		}
		else
			item = elm_genlist_item_append(
					genlist,				// genlist object
					itc,					// item class
					id,						// item class user data
					NULL,
					ELM_GENLIST_ITEM_NONE,	// item type
					gl_selected_cb,				// select smart callback
					ad);					// smart callback user data
		id->item = item;

	}

	elm_genlist_item_class_free(gtc);
	elm_genlist_item_class_free(itc);
	evas_object_show(genlist);

	return genlist;
}

static Eina_Bool
naviframe_pop_cb(void *data, Elm_Object_Item *it)
{
	ui_app_exit();
	return EINA_FALSE;
}

static void
create_base_gui(appdata_s *ad)
{
	Evas_Object *btn, *conform, *genlist;
	Elm_Object_Item *nf_it;

	/* Window */
	ad->win = elm_win_util_standard_add(PACKAGE, PACKAGE);
	elm_win_conformant_set(ad->win, EINA_TRUE);
	elm_win_autodel_set(ad->win, EINA_TRUE);

	evas_object_smart_callback_add(ad->win, "delete,request", win_delete_request_cb, NULL);

	/* Conformant */
	conform = elm_conformant_add(ad->win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(ad->win, ELM_WIN_INDICATOR_OPAQUE);
	elm_win_resize_object_add(ad->win, conform);
	evas_object_show(conform);

	/* Naviframe */
	ad->nf = elm_naviframe_add(conform);
	elm_object_content_set(conform, ad->nf);
	eext_object_event_callback_add(ad->nf, EEXT_CALLBACK_BACK, eext_naviframe_back_cb, NULL);
	eext_object_event_callback_add(ad->nf, EEXT_CALLBACK_MORE, eext_naviframe_more_cb, NULL);
	evas_object_show(ad->nf);

	/* Genlist */
	genlist = create_genlist(ad);

	btn = create_title_btn(ad);
	nf_it = elm_naviframe_item_push(ad->nf, "Email", NULL, NULL, genlist, NULL);
	elm_object_item_part_content_set(nf_it, "title_right_btn", btn);
	elm_naviframe_item_pop_cb_set(nf_it, naviframe_pop_cb, NULL);

	/* Show window after base gui is set up */
	evas_object_show(ad->win);
}

static bool
app_create(void *data)
{
	/* Hook to take necessary actions before main event loop starts
		Initialize UI resources and application's data
		If this function returns true, the main loop of application starts
		If this function returns false, the application is terminated */
	appdata_s *ad = data;

	create_base_gui(ad);

	return true;
}

static void
app_control(app_control_h app_control, void *data)
{
	/* Handle the launch request. */
}

static void
app_pause(void *data)
{
	/* Take necessary actions when application becomes invisible. */
}

static void
app_resume(void *data)
{
	/* Take necessary actions when application becomes visible. */
}

static void
app_terminate(void *data)
{
	/* Release all resources. */
}

static void
ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);
	elm_language_set(locale);
	free(locale);
	return;
}

static void
ui_app_orient_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_DEVICE_ORIENTATION_CHANGED*/
	return;
}

static void
ui_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}

static void
ui_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_BATTERY*/
}

static void
ui_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_MEMORY*/
}

int
main(int argc, char *argv[])
{
	appdata_s ad = {0,};
	int ret = 0;

	ui_app_lifecycle_callback_s event_callback = {0,};
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY], APP_EVENT_LOW_BATTERY, ui_app_low_battery, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY], APP_EVENT_LOW_MEMORY, ui_app_low_memory, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_DEVICE_ORIENTATION_CHANGED], APP_EVENT_DEVICE_ORIENTATION_CHANGED, ui_app_orient_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED], APP_EVENT_REGION_FORMAT_CHANGED, ui_app_region_changed, &ad);
	ui_app_remove_event_handler(handlers[APP_EVENT_LOW_MEMORY]);

	ret = ui_app_main(argc, argv, &event_callback, &ad);
	if (ret != APP_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "app_main() is failed. err = %d", ret);
	}

	return ret;
}
